
av = 'pymilkss 1.3eee'

class dfa():

    def __init__(self, kws=set(), senseless=''):
        '''
        kws: 敏感词
        senseless: 无意义字符库, 在检测过程中会自动忽略.
        '''
        self.tree = {}
        self.senseless = set('''!@#$%^&*()_+`~-=,./<>?;:'"[]{}|''')
        if kws: self.add_kws(kws)
        if senseless: self.add_senseless(senseless)

    def add_kws(self, kws):
        for kw in kws:
            tree = self.tree
            for x in kw:
                tree = tree.setdefault(x, {'__end__': False})
            tree['__end__'] = True

    def add_senseless(self, s):
        self.senseless |= set(s)

    def find_from_index(self, text, start_index, mode='long'):
        ''' 从指定的索引处开始检测敏感词 '''
        indexs = []
        rlong = 0
        kw_size = 0  # 包括特殊字符的敏感词的长度
        tree = self.tree
        for i in range(start_index, len(text)):
            s = text[i]
            if s in tree:
                kw_size += 1
                tree = tree[s]
                if tree["__end__"]:
                    if mode == 'long':
                        rlong = kw_size
                    elif mode == 'short':
                        return [kw_size]
                    elif mode == 'all':
                        indexs.append(kw_size)
                    else:
                        raise Exception("mode err!")
            elif s in self.senseless:
                kw_size += 1
            else:
                break
        return [rlong] if rlong else indexs

    def find(self, text, mode='long'):
        kws = []
        for i, t in enumerate(text):
            if t not in self.senseless:
                indexs = self.find_from_index(text, i, mode=mode)
                for x in indexs: kws.append(text[i: i + x])
        return kws


if __name__ == '__main__':
    kws = {'本月', '本月是', '本月是11月', '下个月', '下个月是12月'}
    senseless = '!@#$%^&*()_+`~-=,./<>?;:'
    text = '_本#月_是_11*月_and_下_个_月_是_1#2_月_'

    dtool = dfa(kws=kws, senseless=senseless)

    print(dtool.find(text, mode='short'))
    # >>> ['本#月', '下_个_月']

    print(dtool.find(text, mode='long'))
    # >>> ['本#月_是_11*月', '下_个_月_是_1#2_月']

    print(dtool.find(text, mode='all'))
    # >>> ['本#月', '本#月_是', '本#月_是_11*月', '下_个_月', '下_个_月_是_1#2_月']
